@@ -1,6 +1,3 @@
-#[allow(dead_code)]
-pub mod io;
-
 pub mod bench_aabb;
 pub mod bench_full;
 pub mod bench_mesh;
