@@ -12,6 +12,10 @@
 //!  In particular it adds `From` impls for the [mesh](crate::mesh) types used by this crate to convert them to
 //!  [`vtkio::model::UnstructuredGridPiece`](https://docs.rs/vtkio/0.6.*/vtkio/model/struct.UnstructuredGridPiece.html) and [`vtkio::model::DataSet`](https://docs.rs/vtkio/0.6.*/vtkio/model/enum.DataSet.html)
 //!  types. If the feature is enabled, The crate exposes its `vtkio` dependency as `splashsurflib::vtkio`.
+//! - **`io`**: Enables the [`io`] module, containing functions to load and store particle and mesh files
+//!  from various file formats, e.g. `VTK`, `OBJ`, `BGEO` etc. This feature implies the `vtk_extras` feature.
+//!  It is disabled by default because a pure "online" surface reconstruction might not need any file IO.
+//!  The feature adds several dependencies to support the file formats.
 //! - **`profiling`**: Enables profiling of internal functions. The resulting data can be displayed using the functions
 //!  from the [`profiling`] module. Furthermore, it exposes the [`profile`] macro that can be used e.g.
 //!  by binary crates calling into this library to add their own profiling scopes to the measurements.
@@ -50,15 +54,20 @@ pub mod profiling_macro;
 mod aabb;
 pub mod density_map;
 pub mod generic_tree;
+#[cfg(feature = "io")]
+#[cfg_attr(doc_cfg, doc(cfg(feature = "io")))]
+pub mod io;
 pub mod kernel;
 pub mod marching_cubes;
 pub mod mesh;
 pub mod neighborhood_search;
 pub mod octree;
 mod reconstruction;
+pub mod sph_interpolation;
 pub mod topology;
 mod traits;
 pub mod uniform_grid;
+#[macro_use]
 mod utils;
 pub(crate) mod workspace;
 
@@ -69,8 +78,6 @@ pub(crate) mod workspace;
 // TODO: Remove anyhow/thiserror from lib?
 // TODO: Write more unit tests (e.g. AABB, UniformGrid, neighborhood search)
 // TODO: Test kernels with property based testing?
-// TODO: Add free particles back again after triangulation as sphere meshes if they were removed
-// TODO: Detect free particles by just comparing with the SPH density of a free particle? (no need for extra neighborhood search?)
 // TODO: More and better error messages with distinct types
 // TODO: Make flat indices strongly typed
 // TODO: Function that detects smallest usable index type
@@ -101,16 +108,6 @@ pub(crate) fn new_map<K, V>() -> MapType<K, V> {
 
 pub(crate) type ParallelMapType<K, V> = dashmap::DashMap<K, V, HashState>;
 
-/// Macro version of Option::map that allows using e.g. using the ?-operator in the map expression
-macro_rules! map_option {
-    ($some_optional:expr, $value_identifier:ident => $value_transformation:expr) => {
-        match $some_optional {
-            Some($value_identifier) => Some($value_transformation),
-            None => None,
-        }
-    };
-}
-
 /// Parameters for the spatial decomposition
 #[derive(Clone, Debug)]
 pub struct SpatialDecompositionParameters<R: Real> {
@@ -167,7 +164,7 @@ pub enum ParticleDensityComputationStrategy {
 }
 
 impl<R: Real> SpatialDecompositionParameters<R> {
-    /// Tries to convert the parameters from one [Real] type to another [Real] type, returns None if conversion fails
+    /// Tries to convert the parameters from one [`Real`] type to another [`Real`] type, returns `None` if conversion fails
     pub fn try_convert<T: Real>(&self) -> Option<SpatialDecompositionParameters<T>> {
         Some(SpatialDecompositionParameters {
             subdivision_criterion: self.subdivision_criterion.clone(),
@@ -225,10 +222,12 @@ impl<R: Real> Parameters<R> {
 pub struct SurfaceReconstruction<I: Index, R: Real> {
     /// Background grid that was used as a basis for generating the density map for marching cubes
     grid: UniformGrid<I, R>,
-    /// Octree built for domain decomposition
+    /// Octree constructed for domain decomposition
     octree: Option<Octree<I, R>>,
     /// Point-based density map generated from the particles that was used as input to marching cubes
     density_map: Option<DensityMap<I, R>>,
+    /// Per particle densities
+    particle_densities: Option<Vec<R>>,
     /// Surface mesh that is the result of the surface reconstruction
     mesh: TriMesh3d<R>,
     /// Workspace with allocated memory for subsequent surface reconstructions
@@ -242,6 +241,7 @@ impl<I: Index, R: Real> Default for SurfaceReconstruction<I, R> {
             grid: UniformGrid::new_zero(),
             octree: None,
             density_map: None,
+            particle_densities: None,
             mesh: TriMesh3d::default(),
             workspace: ReconstructionWorkspace::default(),
         }
@@ -254,17 +254,22 @@ impl<I: Index, R: Real> SurfaceReconstruction<I, R> {
         &self.mesh
     }
 
-    /// Returns a reference to the octree generated for spatial decomposition of the input particles
+    /// Returns a reference to the octree generated for spatial decomposition of the input particles (mostly useful for debugging visualization)
     pub fn octree(&self) -> Option<&Octree<I, R>> {
         self.octree.as_ref()
     }
 
-    /// Returns a reference to the sparse density map (discretized on the vertices of the background grid) that is used as input for marching cubes
+    /// Returns a reference to the sparse density map (discretized on the vertices of the background grid) that is used as input for marching cubes (always `None` when using domain decomposition)
     pub fn density_map(&self) -> Option<&DensityMap<I, R>> {
         self.density_map.as_ref()
     }
 
-    /// Returns a reference to the virtual background grid that was used as a basis for discretization of the density map for marching cubes, can be used to convert the density map to a hex mesh (using [sparse_density_map_to_hex_mesh](density_map::sparse_density_map_to_hex_mesh))
+    /// Returns a reference to the global particle density vector if it was computed during the reconstruction (always `None` when using independent subdomains with domain decomposition)
+    pub fn particle_densities(&self) -> Option<&Vec<R>> {
+        self.particle_densities.as_ref()
+    }
+
+    /// Returns a reference to the virtual background grid that was used as a basis for discretization of the density map for marching cubes, can be used to convert the density map to a hex mesh (using [`density_map::sparse_density_map_to_hex_mesh`])
     pub fn grid(&self) -> &UniformGrid<I, R> {
         &self.grid
     }
