@@ -17,6 +17,7 @@
 //!  - `Into<DataSet>` implementations for the basic mesh types
 
 use crate::{new_map, Real};
+use bytemuck_derive::{Pod, Zeroable};
 use nalgebra::{Unit, Vector3};
 use rayon::prelude::*;
 use std::cell::RefCell;
@@ -31,7 +32,7 @@ use vtkio::model::{Attribute, DataSet, UnstructuredGridPiece};
 #[derive(Clone, Debug)]
 pub struct MeshAttribute<R: Real> {
     /// Name of the attribute
-    pub name: &'static str,
+    pub name: String,
     /// Data of the attribute
     pub data: AttributeData<R>,
 }
@@ -112,12 +113,15 @@ pub trait CellConnectivity {
 }
 
 /// Cell type for the [`TriMesh3d`]
+#[derive(Copy, Clone, Pod, Zeroable)]
 #[repr(transparent)]
 pub struct TriangleCell(pub [usize; 3]);
 /// Cell type for the [`HexMesh3d`]
+#[derive(Copy, Clone, Pod, Zeroable)]
 #[repr(transparent)]
 pub struct HexCell(pub [usize; 8]);
 /// Cell type for the [`PointCloud3d`]
+#[derive(Copy, Clone, Pod, Zeroable)]
 #[repr(transparent)]
 pub struct PointCell(pub usize);
 
@@ -159,7 +163,7 @@ impl<R: Real> Mesh3d<R> for TriMesh3d<R> {
     }
 
     fn cells(&self) -> &[TriangleCell] {
-        unsafe { std::mem::transmute::<&[[usize; 3]], &[TriangleCell]>(self.triangles.as_slice()) }
+        bytemuck::cast_slice::<[usize; 3], TriangleCell>(self.triangles.as_slice())
     }
 }
 
@@ -171,7 +175,7 @@ impl<R: Real> Mesh3d<R> for HexMesh3d<R> {
     }
 
     fn cells(&self) -> &[HexCell] {
-        unsafe { std::mem::transmute::<&[[usize; 8]], &[HexCell]>(self.cells.as_slice()) }
+        bytemuck::cast_slice::<[usize; 8], HexCell>(self.cells.as_slice())
     }
 }
 
@@ -183,7 +187,7 @@ impl<R: Real> Mesh3d<R> for PointCloud3d<R> {
     }
 
     fn cells(&self) -> &[PointCell] {
-        unsafe { std::mem::transmute::<&[usize], &[PointCell]>(self.indices.as_slice()) }
+        bytemuck::cast_slice::<usize, PointCell>(self.indices.as_slice())
     }
 }
 
@@ -311,13 +315,8 @@ impl<R: Real> TriMesh3d<R> {
 
         // First, compute the directions of the normals...
         {
-            let normal_directions = unsafe {
-                // This is sound, as Unit<T> has repr(transparent)
-                let vector3_ptr = normals.as_mut_ptr() as *mut Vector3<R>;
-                let normal_directions: &'a mut _ =
-                    std::slice::from_raw_parts_mut(vector3_ptr, normals.len());
-                normal_directions
-            };
+            let normal_directions =
+                bytemuck::cast_slice_mut::<Unit<Vector3<R>>, Vector3<R>>(normals);
             self.vertex_normal_directions_inplace_assume_zeroed(normal_directions);
         }
 
@@ -335,13 +334,8 @@ impl<R: Real> TriMesh3d<R> {
 
         // First, compute the directions of the normals...
         {
-            let normal_directions = unsafe {
-                // This is sound, as Unit<T> has repr(transparent)
-                let vector3_ptr = normals.as_mut_ptr() as *mut Vector3<R>;
-                let normal_directions: &'a mut _ =
-                    std::slice::from_raw_parts_mut(vector3_ptr, normals.len());
-                normal_directions
-            };
+            let normal_directions =
+                bytemuck::cast_slice_mut::<Unit<Vector3<R>>, Vector3<R>>(normals);
             self.par_vertex_normal_directions_inplace_assume_zeroed(normal_directions);
         }
 
@@ -527,25 +521,25 @@ impl<R: Real, MeshT: Mesh3d<R>> MeshWithData<R, MeshT> {
 
 impl<R: Real> MeshAttribute<R> {
     /// Creates a new named mesh attribute with the given data
-    pub fn new(name: &'static str, data: impl Into<AttributeData<R>>) -> Self {
+    pub fn new<S: Into<String>>(name: S, data: impl Into<AttributeData<R>>) -> Self {
         Self {
-            name,
+            name: name.into(),
             data: data.into(),
         }
     }
 
     /// Creates a new named mesh attribute with scalar values implementing the [`Real`](crate::Real) trait
-    pub fn new_real_scalar(name: &'static str, data: impl Into<Vec<R>>) -> Self {
+    pub fn new_real_scalar<S: Into<String>>(name: S, data: impl Into<Vec<R>>) -> Self {
         Self {
-            name,
+            name: name.into(),
             data: AttributeData::ScalarReal(data.into()),
         }
     }
 
     /// Creates a new named mesh attribute with scalar values implementing the [`Real`](crate::Real) trait
-    pub fn new_real_vector3(name: &'static str, data: impl Into<Vec<Vector3<R>>>) -> Self {
+    pub fn new_real_vector3<S: Into<String>>(name: S, data: impl Into<Vec<Vector3<R>>>) -> Self {
         Self {
-            name,
+            name: name.into(),
             data: AttributeData::Vector3Real(data.into()),
         }
     }
@@ -556,12 +550,12 @@ impl<R: Real> MeshAttribute<R> {
     fn to_vtk_attribute(&self) -> Attribute {
         match &self.data {
             AttributeData::ScalarU64(u64_vec) => {
-                Attribute::scalars(self.name, 1).with_data(u64_vec.clone())
+                Attribute::scalars(&self.name, 1).with_data(u64_vec.clone())
             }
             AttributeData::ScalarReal(real_vec) => {
-                Attribute::scalars(self.name, 1).with_data(real_vec.clone())
+                Attribute::scalars(&self.name, 1).with_data(real_vec.clone())
             }
-            AttributeData::Vector3Real(vec3r_vec) => Attribute::scalars(self.name, 3)
+            AttributeData::Vector3Real(vec3r_vec) => Attribute::scalars(&self.name, 3)
                 .with_data(vec3r_vec.iter().flatten().copied().collect::<Vec<R>>()),
         }
     }
