@@ -316,7 +316,7 @@ impl<I: Index, R: Real> Octree<I, R> {
         });
 
         assert_eq!(mesh.cells.len(), ids.len());
-        MeshWithData::new(mesh).with_cell_data(MeshAttribute::new("node_id", ids))
+        MeshWithData::new(mesh).with_cell_data(MeshAttribute::new("node_id".to_string(), ids))
     }
 }
 
@@ -833,7 +833,7 @@ impl<I: Index, R: Real> NodeData<I, R> {
     }
 }
 
-/// Returns the [PointIndex] of the octree subdivision point for an [OctreeNode] with the given lower and upper points
+/// Returns the [`PointIndex`] of the octree subdivision point for an [`OctreeNode`] with the given lower and upper points
 fn get_split_point<I: Index, R: Real>(
     grid: &UniformGrid<I, R>,
     lower: &PointIndex<I>,
