@@ -1,7 +1,5 @@
-pub mod io;
-
-#[cfg(feature = "vtk_extras")]
+#[cfg(feature = "io")]
 pub mod test_full;
 pub mod test_neighborhood_search;
-#[cfg(feature = "vtk_extras")]
+#[cfg(feature = "io")]
 pub mod test_octree;
